@@ -11,20 +11,23 @@ from pathlib import Path
 import asyncio
 import shutil
 from time import perf_counter
-from typing import Iterable, Set, Tuple
+from typing import Callable, Iterable, Optional, Set, Tuple
 from dataclasses import dataclass
 from tqdm import tqdm
 from aiohttp import ClientSession, ClientTimeout, TCPConnector, ClientResponseError, InvalidURL
+
+
 @dataclass
 class _FormsiteDownloader:
 
     """Handles the distribution of downlaod tasks across DownloadWorkers."""
+
     download_folder: str
     links: Iterable[str]
     max_concurrent_downloads: int = 10
     timeout: int = 80
     retries: int = 1
-    filename_regex: str = r''
+    filename_regex: str = r""
     strip_prefix: bool = False
     overwrite_existing: bool = True
     report_downloads: bool = False
@@ -38,7 +41,7 @@ class _FormsiteDownloader:
         self.dl_queue = asyncio.Queue()
         if not self.overwrite_existing and len(self.links) > 0:
             for element in self.links:
-                url = element.rsplit('/', 1)[0]+"/"
+                url = element.rsplit("/", 1)[0] + "/"
                 break
             filenames_in_dl_dir = self._list_files_in_download_dir(url)
             self.links = set(self.links) - filenames_in_dl_dir
@@ -48,31 +51,20 @@ class _FormsiteDownloader:
 
     async def Start(self) -> None:
         """Starts download of links."""
-        pbar = tqdm(total=len(self.links),
-                    desc="Downloading files",
-                    unit='f',
-                    leave=False,
-                    dynamic_ncols=True,
-                    ncols=80) if self.display_progress else None
+        pbar = tqdm(total=len(self.links), desc="Downloading files", unit="f", leave=False, dynamic_ncols=True, ncols=80) if self.display_progress else None
         os.makedirs(self.download_folder, exist_ok=True)
         async with ClientSession(connector=TCPConnector(limit=0)) as session:
-            self.internal_state.update_pbar_callback = pbar.update if pbar is not None else None
+            self.internal_state.update_pbar_callback = pbar.update if pbar else None
             for link in self.links:
                 self.dl_queue.put_nowait((link, 0))
-            tasks = [asyncio.ensure_future(
-                DownloadWorker(self.download_folder,
-                               self.dl_queue,
-                               self.semaphore,
-                               session,
-                               self.internal_state,
-                               timeout=self.timeout,
-                               retries=self.retries,
-                               pbar=pbar,
-                               filename_regex=self.filename_regex,
-                               strip_prefix=self.strip_prefix).main())
-                     for _ in range(self.max_concurrent_downloads)]
+            tasks = [
+                asyncio.ensure_future(
+                    DownloadWorker(self.download_folder, self.dl_queue, self.semaphore, session, self.internal_state, timeout=self.timeout, retries=self.retries, pbar=pbar, filename_regex=self.filename_regex, strip_prefix=self.strip_prefix).main()
+                )
+                for _ in range(self.max_concurrent_downloads)
+            ]
             await asyncio.gather(*tasks)
-        pbar.close()
+        pbar.close() if pbar else None
         if self.internal_state.failed > 0 and self.write_failed:
             self.internal_state.write_failed()
         if self.report_downloads:
@@ -82,13 +74,15 @@ class _FormsiteDownloader:
         """Lists all files in `self.download_folder`, inserts `url` before the filename."""
         filenames_in_dir = set()
         for file in os.listdir(self.download_folder):
-            filenames_in_dir.add(url+file)
+            filenames_in_dir.add(url + file)
         return filenames_in_dir
 
+
 @dataclass
 class DownloadWorkerState:
 
     """Contains methods that track internal downlaod progressa cross downloader workers."""
+
     urls: Iterable[str]
     active_workers: int
 
@@ -102,31 +96,30 @@ class DownloadWorkerState:
         self.failed_urls: set = set()
         self.success_urls: set = set()
         self.complete_urls: list = list()
-        self.update_pbar_callback = None
+        self.update_pbar_callback: Optional[Callable] = None
         self.last_progress_display_update = 0
 
     def total_complete(self):
         """Returns sum of `self.success` and `self.failed`"""
         return self.success + self.failed
 
-    def write_failed(self, target: str = './failed_downloads.txt'):
+    def write_failed(self, target: str = "./failed_downloads.txt"):
         """Writes contents of `self.failed_urls` to a file."""
-        with open(target, 'a', encoding='utf-8') as writer:
+        with open(target, "a", encoding="utf-8") as writer:
             to_write = self.get_failed_url_diff()
             for i in to_write:
                 writer.write(f"{i}\n")
-        print(f"{len(to_write)} failures, please see failed_downloads.txt for more info"
-              "\nfor timeout errors, try increasing timeout or reducing max concurrent downloads")
+        print(f"{len(to_write)} failures, please see failed_downloads.txt for more info" "\nfor timeout errors, try increasing timeout or reducing max concurrent downloads")
 
-    def write_all(self, target: str = './downloads_status.txt'):
+    def write_all(self, target: str = "./downloads_status.txt"):
         """Writes contents of `self.complete_urls` to a file."""
-        with open(target, 'w', encoding='utf-8') as writer:
+        with open(target, "w", encoding="utf-8") as writer:
             for i in self.complete_urls:
                 writer.write(f"{i}\n")
 
     def display_relevant_info(self):
         """Debug: prints some relevant internal state values"""
-        print('----------------------------------')
+        print("----------------------------------")
         print(f"enqueued: {self.enqueued}")
         print(f"active workers: {self.active_workers}")
         print(f"files in progress: {self.in_progress}")
@@ -141,14 +134,16 @@ class DownloadWorkerState:
             return False
         elif self.active_workers > self.in_progress:
             return True
-        else: return False
+        else:
+            return False
 
     def can_terminate_worker(self) -> bool:
         """Decrements allowed number of active workers if conditions allow it."""
         if self.try_exit():
             self.active_workers -= 1
             return True
-        else: return False
+        else:
+            return False
 
     def mark_success(self, url: str) -> None:
         """Increments internal counter to match completed downloads."""
@@ -179,12 +174,14 @@ class DownloadWorkerState:
         """Returns a difference between all input URLs and successfully downloaded urls."""
         return set(self.urls) - set(self.success_urls)
 
+
 @dataclass
 class DownloadWorker:
 
     """download_folder is a path to download directory,
     \nqueue, semaphore, session, internal state and pbar are shared across all workers
     """
+
     download_folder: str
     queue: asyncio.Queue
     semaphore: asyncio.Semaphore
@@ -193,7 +190,7 @@ class DownloadWorker:
     timeout: int = 80
     retries: int = 1
     pbar: tqdm = None
-    filename_regex: str = r''
+    filename_regex: str = r""
     strip_prefix: bool = False
 
     def __post_init__(self):
@@ -209,7 +206,7 @@ class DownloadWorker:
                     break
                 url, attempt = await self.queue.get()
                 await self.semaphore.acquire()
-                if abs(self.internal_state.last_progress_display_update - perf_counter()) > 5: # seconds
+                if abs(self.internal_state.last_progress_display_update - perf_counter()) > 5:  # seconds
                     self._update_pbar(desc="Downloading files")
                 self.internal_state.start_iteration()
                 try:
@@ -225,29 +222,19 @@ class DownloadWorker:
                 self.internal_state.end_iteration()
         return 0
 
-    async def _fetch(self, url: str, filename: str, target: str, chunk_size: int = 4*1024,
-                     in_progress_ext: str = '.tmp') -> int:
+    async def _fetch(self, url: str, filename: str, target: str, chunk_size: int = 4 * 1024, in_progress_ext: str = ".tmp") -> int:
         """The core download function with `session.get` request."""
-        display_name = filename[:20]+"…"+filename[-5:] if len(filename) > 25 else filename[:25]
+        display_name = filename[:20] + "…" + filename[-5:] if len(filename) > 25 else filename[:25]
         async with self.session.get(url, timeout=self.client_timeout) as response:
             response.raise_for_status()
-            pbar = tqdm(desc=display_name,
-                        total=response.content_length,
-                        leave=False,
-                        unit='b',
-                        unit_scale=True,
-                        unit_divisor=1024,
-                        ncols=80) if self.display_progress else None
-            with open(target+in_progress_ext, "wb") as f:
+            pbar = tqdm(desc=display_name, total=response.content_length, leave=False, unit="b", unit_scale=True, unit_divisor=1024, ncols=80) if self.display_progress else None
+            with open(target + in_progress_ext, "wb") as f:
                 async for chunk in response.content.iter_chunked(chunk_size):
                     f.write(chunk)
-                    if pbar is not None:
-                        pbar.update(len(chunk))
-        try:
-            pbar.close()
-        except AttributeError:
-            pass
-        shutil.move(target+in_progress_ext, target)
+                    pbar.update(len(chunk)) if pbar else None
+
+        pbar.close() if pbar else None
+        shutil.move(target + in_progress_ext, target)
         return 0
 
     async def _download(self, url: str) -> int:
@@ -260,11 +247,11 @@ class DownloadWorker:
         """Decides what errors cancel downloads and which are retried."""
         if isinstance(exception, ClientResponseError):
             if exception.status == 403:
-                self._update_pbar(desc=f"{exception}"[:79]+"…")
+                self._update_pbar(desc=f"{exception}"[:79] + "…")
                 self.internal_state.last_progress_display_update = perf_counter()
                 self.internal_state.mark_fail(url, exception)
             elif exception.status == 404:
-                self._update_pbar(desc=f"{exception}"[:79]+"…")
+                self._update_pbar(desc=f"{exception}"[:79] + "…")
                 self.internal_state.last_progress_display_update = perf_counter()
                 self.internal_state.mark_fail(url, exception)
             else:
@@ -274,13 +261,13 @@ class DownloadWorker:
         elif attempt < self.retries:
             self._retry_dl(url, attempt)
         else:
-            self._update_pbar(desc=f"Failed {url}"[:79]+"…")
+            self._update_pbar(desc=f"Failed {url}"[:79] + "…")
             self.internal_state.last_progress_display_update = perf_counter()
             self.internal_state.mark_fail(url, exception)
 
     def _retry_dl(self, url: str, attempt: int) -> None:
         """Puts failed downloads to the end of queue, if attempt < max retry."""
-        self._update_pbar(desc=f"Retrying ({attempt}/{self.retries}) download of {url}"[:79]+"…")
+        self._update_pbar(desc=f"Retrying ({attempt}/{self.retries}) download of {url}"[:79] + "…")
         self.internal_state.last_progress_display_update = perf_counter()
         attempt += 1
         self.internal_state.enqueued += 1
@@ -295,8 +282,8 @@ class DownloadWorker:
         """Gets filename from url. Returns filename and path+filename as target."""
         filename = f"{url.split('/')[-1:][0]}"
         if self.strip_prefix:
-            filename = re.sub(r'^f-[\d]*-[\d]*-', r'', filename)
-        if self.filename_compiled_regex.pattern != '':
+            filename = re.sub(r"^f-[\d]*-[\d]*-", r"", filename)
+        if self.filename_compiled_regex.pattern != "":
             filename = self._regex_substitution(filename, self.filename_compiled_regex)
             target = f"{self.download_folder}/{filename}"
             target = self._check_if_file_exists(target)
@@ -308,20 +295,20 @@ class DownloadWorker:
         """If `overwrite_downloads is False`, checks which URLs to omit based on filenames."""
         path = Path(filename).resolve()
         if path.exists():
-            temp = filename.rsplit('.', 1)
-            if temp[0].endswith(f'_{appended_number}'):
-                temp[0] = temp[0][:temp[0].rfind(f'_{appended_number}')]
+            temp = filename.rsplit(".", 1)
+            if temp[0].endswith(f"_{appended_number}"):
+                temp[0] = temp[0][: temp[0].rfind(f"_{appended_number}")]
             try:
                 filename = temp[0] + f"_{appended_number+1}." + temp[1]
             except IndexError:
                 filename = temp[0] + f"_{appended_number+1}"
-            filename = self._check_if_file_exists(filename, appended_number=appended_number+1)
+            filename = self._check_if_file_exists(filename, appended_number=appended_number + 1)
         return filename
 
     def _regex_substitution(self, filename: str, filename_regex: re.Pattern) -> str:
         """Removes characters that match regex."""
         try:
-            old_filename_tuple = filename.rsplit('.', 1)
+            old_filename_tuple = filename.rsplit(".", 1)
             try:
                 new_filename = f"{filename_regex.sub('', old_filename_tuple[0])}.{old_filename_tuple[1]}"
             except:
