@@ -24,8 +24,10 @@ from .processing import _FormsiteProcessing
 from .api import _FormsiteAPI
 from .auth import FormsiteCredentials
 from tqdm import tqdm
-__version__ = '1.3.19'
-__author__ = 'jakub.strnad@protonmail.com'
+
+__version__ = "1.3.20"
+__author__ = "jakub.strnad@protonmail.com"
+
 
 def _shift_param_date(date: Union[str, dt], timezone_offset: td) -> str:
     """Shifts input date in the string format/datetime by timedelta in timezone offset.
@@ -60,6 +62,7 @@ def _shift_param_date(date: Union[str, dt], timezone_offset: td) -> str:
 
     return dt.strftime(date, "%Y-%m-%dT%H:%M:%SZ")
 
+
 def _extract_timezone_from_str(timezone: str, timezone_re: str) -> td:
     """Parses input timezone.
 
@@ -72,16 +75,17 @@ def _extract_timezone_from_str(timezone: str, timezone_re: str) -> td:
     """
     tz_offset = None
     if re.search(timezone_re, timezone) is not None:
-        tz_str = timezone.replace(r'\'', '').replace(r'\"','')
-        if ':' in tz_str:
-            tz_tuple = tz_str.split(':', 1)
+        tz_str = timezone.replace(r"\'", "").replace(r"\"", "")
+        if ":" in tz_str:
+            tz_tuple = tz_str.split(":", 1)
         else:
-            t = (len(tz_str) - 2)
+            t = len(tz_str) - 2
             tz_tuple = (tz_str[:t], tz_str[-2:])
 
-        tz_offset = td(hours=int(tz_tuple[0]), seconds=int(tz_tuple[1])/60)
+        tz_offset = td(hours=int(tz_tuple[0]), seconds=int(tz_tuple[1]) / 60)
     return tz_offset
 
+
 def _calculate_tz_offset(timezone: str) -> Tuple[td, td, dt]:
     """Calculates timezone offset relative to input TARGET timezone string UTC.
 
@@ -98,10 +102,10 @@ def _calculate_tz_offset(timezone: str) -> Tuple[td, td, dt]:
     utc_date = dt.utcnow()
     diff_local_utc = local_date - utc_date
     offset_utc = local_date - utc_date
-    if timezone == 'local':
+    if timezone == "local":
         offset_local = td(seconds=0)
     else:
-        tz_input_regex = r'(\+|\-|)([0-1]\d[0-5]\d|[0-1]\d\:[0-5]\d|\d\:[0-5]\d)'
+        tz_input_regex = r"(\+|\-|)([0-1]\d[0-5]\d|[0-1]\d\:[0-5]\d|\d\:[0-5]\d)"
         # parses the following formats:
         # America/Chicago
         # +1200
@@ -112,17 +116,18 @@ def _calculate_tz_offset(timezone: str) -> Tuple[td, td, dt]:
         offset_local = _extract_timezone_from_str(timezone, tz_input_regex)
 
         if offset_local is None:
-            if re.search(r'\w+/\w+', timezone) is not None:
+            if re.search(r"\w+/\w+", timezone) is not None:
                 off_local = pytztimezone(timezone).localize(local_date).strftime("%z")
-                t = (len(off_local) - 2)
+                t = len(off_local) - 2
                 l_inp = (off_local[:t], off_local[-2:])
-                offset_utc = td(hours=int(l_inp[0]), seconds=int(l_inp[1])/60)
+                offset_utc = td(hours=int(l_inp[0]), seconds=int(l_inp[1]) / 60)
                 offset_local = offset_utc - diff_local_utc
             else:
                 raise UnknownTimeZoneError(timezone)
 
     return offset_local, offset_utc, local_date
 
+
 def _validate_path(path: str) -> str:
     """Converts input path into POSIX format. Creates parent directories if necessary.
 
@@ -141,6 +146,7 @@ def _validate_path(path: str) -> str:
 
     return output_file.as_posix()
 
+
 @dataclass
 class FormsiteParams:
 
@@ -155,19 +161,20 @@ class FormsiteParams:
     `resultslabels` and `resultsview` More info on Formsite website or FS API of your specific form.\n
     `sort` ( "asc" | "desc" ) sorts results by reference number in ascending or descending order.
     """
+
     last: Optional[int] = None
     afterref: Optional[int] = None
     beforeref: Optional[int] = None
     afterdate: Optional[Union[str, dt]] = None
     beforedate: Optional[Union[str, dt]] = None
-    timezone: Optional[str] = 'local'
+    timezone: Optional[str] = "local"
     resultslabels: Optional[int] = None
     resultsview: Optional[int] = 11
     sort: Optional[str] = "desc"
 
     def __post_init__(self):
         """Calls `_calculate_tz_offset` internal function."""
-        self.timezone = 'local' if self.timezone is None else self.timezone
+        self.timezone = "local" if self.timezone is None else self.timezone
         self.timezone_offset, self.timezone_offset_utc, self.local_datetime = _calculate_tz_offset(self.timezone)
         self.filters = dict()
 
@@ -181,20 +188,20 @@ class FormsiteParams:
             dict: params dict
         """
         results_params: dict[str, Union[str, int]] = dict()
-        results_params['page'] = 1
-        results_params['limit'] = single_page_limit
+        results_params["page"] = 1
+        results_params["limit"] = single_page_limit
         if self.afterref is not None:
-            results_params['after_id'] = self.afterref
+            results_params["after_id"] = self.afterref
         if self.beforeref is not None:
-            results_params['before_id'] = self.beforeref
+            results_params["before_id"] = self.beforeref
         if self.afterdate is not None:
             parsed_afterdate = _shift_param_date(self.afterdate, self.timezone_offset)
-            results_params['after_date'] = parsed_afterdate
+            results_params["after_date"] = parsed_afterdate
         if self.beforedate is not None:
             parsed_beforedate = _shift_param_date(self.beforedate, self.timezone_offset)
-            results_params['before_date'] = parsed_beforedate
-        if self.resultsview is not None: # 11 = all items + statistics results view
-            results_params['results_view'] = self.resultsview
+            results_params["before_date"] = parsed_beforedate
+        if self.resultsview is not None:  # 11 = all items + statistics results view
+            results_params["results_view"] = self.resultsview
 
         results_params.update(self.filters)
         return results_params
@@ -212,18 +219,18 @@ class FormsiteParams:
         Raises:
             Exception: Entered invalid search type.
         """
-        valid_types = {'equals', 'contains', 'begins', 'ends'}
+        valid_types = {"equals", "contains", "begins", "ends"}
         if search_type in valid_types:
-            if search_type == 'equals':
+            if search_type == "equals":
                 self.filters.update({f"search_equals[{col}]": search_value})
-            elif search_type == 'contains':
+            elif search_type == "contains":
                 self.filters.update({f"search_contains[{col}]": search_value})
-            elif search_type == 'begins':
+            elif search_type == "begins":
                 self.filters.update({f"search_begins[{col}]": search_value})
-            elif search_type == 'ends':
+            elif search_type == "ends":
                 self.filters.update({f"search_ends[{col}]": search_value})
         else:
-            raise Exception(f'Invalid search type entered. Must be one of {valid_types}.')
+            raise ValueError(f"Invalid search type entered. Must be one of {valid_types}.")
 
     def set_filter_method(self, method: str):
         """How to combine multiple search criteria.
@@ -236,11 +243,11 @@ class FormsiteParams:
         Raises:
             Exception: Entered invalid method.
         """
-        valid_methods = {'and', 'or'}
+        valid_methods = {"and", "or"}
         if method in valid_methods:
             self.filters.update({"search_method": method})
         else:
-            raise Exception(f'Invalid search method entered. Must be one of {valid_methods}.')
+            raise ValueError(f"Invalid search method entered. Must be one of {valid_methods}.")
 
     def get_items_as_dict(self) -> dict:
         """Returns a dict that gets parsed as parameters by aiohttp when making a request."""
@@ -251,6 +258,7 @@ class FormsiteParams:
         else:
             raise ValueError(f"resultslabels must be an int, got '{self.resultslabels}'")
 
+
 @dataclass
 class FormsiteInterface:
     """A base class for interacting with the formsite API.
@@ -290,6 +298,7 @@ class FormsiteInterface:
     Returns:
         FormsiteInterface: An instance of the FormsiteInterface class.
     """
+
     form_id: str
     auth: FormsiteCredentials
     params: FormsiteParams = FormsiteParams()
@@ -322,7 +331,7 @@ class FormsiteInterface:
         """Allows use of context managers."""
         del self
 
-    def fetch_raw(self, get_items: bool = True, get_results: bool = True, short_delay:float=5.0, long_delay:float=15.0) -> Tuple[dict, List[dict]]:
+    def fetch_raw(self, get_items: bool = True, get_results: bool = True, short_delay: float = 5.0, long_delay: float = 15.0) -> Tuple[dict, List[dict]]:
         """Performs calls to formsite REST API.
 
         Args:
@@ -334,12 +343,7 @@ class FormsiteInterface:
         Returns:
             Tuple[dict, List[dict]]: items_json, List of results_json
         """
-        api_handler = _FormsiteAPI(form_id=self.form_id,
-                                   params=self.params,
-                                   auth=self.auth,
-                                   display_progress=self.display_progress,
-                                   short_delay=short_delay,
-                                   long_delay=long_delay)
+        api_handler = _FormsiteAPI(form_id=self.form_id, params=self.params, auth=self.auth, display_progress=self.display_progress, short_delay=short_delay, long_delay=long_delay)
 
         return api_handler.Start(get_items=get_items, get_results=get_results)
 
@@ -354,14 +358,8 @@ class FormsiteInterface:
         Returns:
             pd.DataFrame: DataFrame of the export.
         """
-        sort_asc = self.params.sort != 'desc'
-        processing_handler = _FormsiteProcessing(items=items,
-                                                 results=results,
-                                                 timezone_offset=self.params.timezone_offset_utc,
-                                                 sort_asc=sort_asc,
-                                                 use_resultslabels=use_resultslabels,
-                                                 display_progress=self.display_progress,
-                                                 params_last=self.params.last)
+        sort_asc = self.params.sort != "desc"
+        processing_handler = _FormsiteProcessing(items=items, results=results, timezone_offset=self.params.timezone_offset_utc, sort_asc=sort_asc, use_resultslabels=use_resultslabels, display_progress=self.display_progress, params_last=self.params.last)
         return processing_handler.Process()
 
     def FetchResults(self, use_resultslabels: bool = True) -> None:
@@ -408,18 +406,18 @@ class FormsiteInterface:
         _ = pbar.update(1) if pbar is not None else None
         try:
             for url_raw in pattern.findall(x):
-                if '|' in url_raw:
-                    for part in url_raw.split(' | '):
+                if "|" in url_raw:
+                    for part in url_raw.split(" | "):
                         if links_filter_pattern.search(url_raw) is not None:
                             links_set.add(part)
                 else:
                     if links_filter_pattern.search(url_raw) is not None:
                         links_set.add(url_raw)
         except Exception as ex:
-            #print(ex)
+            # print(ex)
             pass
 
-    def ExtractLinks(self, links_filter_re: str = r'.+') -> None:
+    def ExtractLinks(self, links_filter_re: str = r".+") -> None:
         """Stores a set of links in `self.Links` of files saved on formsite servers, that were submitted to the specified form.
 
         Args:
@@ -427,20 +425,16 @@ class FormsiteInterface:
         """
         if self.Data is None:
             self.FetchResults()
-        links_re = fr'(https\:\/\/{self.auth.server}\.formsite\.com\/{self.auth.directory}\/files\/.*)'
+        links_re = fr"(https\:\/\/{self.auth.server}\.formsite\.com\/{self.auth.directory}\/files\/.*)"
         url_pattern = re.compile(links_re)
         filter_pattern = re.compile(links_filter_re)
-        self.Links =set()
+        self.Links = set()
         if self.display_progress:
-            pbar = tqdm(desc='Extracting download links', leave=False, unit=' cells')
+            pbar = tqdm(desc="Extracting download links", leave=False, unit=" cells")
         else:
             pbar = None
 
-        _ = self.Data.applymap(lambda x: self._xtract(x,
-                                                      pattern=url_pattern,
-                                                      links_set=self.Links,
-                                                      links_filter_pattern=filter_pattern,
-                                                      pbar=pbar))
+        _ = self.Data.applymap(lambda x: self._xtract(x, pattern=url_pattern, links_set=self.Links, links_filter_pattern=filter_pattern, pbar=pbar))
         try:
             pbar.close()
         except AttributeError:
@@ -452,7 +446,7 @@ class FormsiteInterface:
         while number >= 1024:
             number = number / 1024
             reductions += 1
-        unit = {0: '', 1:'K', 2:'M', 3:'G', 4:'T', 5:'P', 6:'E'}
+        unit = {0: "", 1: "K", 2: "M", 3: "G", 4: "T", 5: "P", 6: "E"}
         return f"{number:0.2f} {unit.get(reductions, None)}B"
 
     def _list_all_forms(self) -> pd.DataFrame:
@@ -463,22 +457,19 @@ class FormsiteInterface:
         """
         with requests.get(self.url_forms, headers=self.auth.get_auth_header()) as response:
             response.raise_for_status()
-            all_forms_json = response.json()['forms']
+            all_forms_json = response.json()["forms"]
             # un-nest the stats object
             for row in all_forms_json:
                 for val in row["stats"]:
-                    row[val] = row['stats'][val]
+                    row[val] = row["stats"][val]
             for row in all_forms_json:
                 for val in row["publish"]:
-                    row[val] = row['publish'][val]
-            forms_df = pd.DataFrame(all_forms_json,columns=['name', 'state', 'directory', 'resultsCount', 'filesSize', 'embed_code', 'link'])
-            forms_df['form_id'] = forms_df.pop('directory')
+                    row[val] = row["publish"][val]
+            forms_df = pd.DataFrame(all_forms_json, columns=["name", "state", "directory", "resultsCount", "filesSize", "embed_code", "link"])
+            forms_df["form_id"] = forms_df.pop("directory")
             return forms_df
 
-    def ListAllForms(self,
-                     sort_by: str = 'name',
-                     display: bool = False,
-                     save2csv: Union[str, bool] = False) -> pd.DataFrame:
+    def ListAllForms(self, sort_by: str = "name", display: bool = False, save2csv: Union[str, bool] = False) -> pd.DataFrame:
         """Prints name, id, results count, filesize and status of all forms into console or csv.
         You can sort in descending order by `name` `form_id` `resultsCount` `filesSize`.
 
@@ -492,40 +483,37 @@ class FormsiteInterface:
         """
         forms_df = self._list_all_forms()
         if display:
-            pd.set_option('display.max_rows', None)
-            pd.set_option('display.max_columns', None)
-            pd.set_option('display.width', None)
-            pd.set_option('display.max_colwidth', 42) # ensures width < 80 cols
-            forms_df = forms_df[['name','resultsCount','filesSize','form_id']]
+            pd.set_option("display.max_rows", None)
+            pd.set_option("display.max_columns", None)
+            pd.set_option("display.width", None)
+            pd.set_option("display.max_colwidth", 42)  # ensures width < 80 cols
+            forms_df = forms_df[["name", "resultsCount", "filesSize", "form_id"]]
             forms_df.sort_values(by=[sort_by], inplace=True, ascending=False, ignore_index=True)
             forms_df.reset_index(drop=True, inplace=True)
-            forms_df = forms_df.append({'name':'Total:', 'resultsCount':forms_df['resultsCount'].sum(), 'filesSize':forms_df['filesSize'].sum(),'form_id':f'{forms_df.shape[0]} forms'}, ignore_index=True)
-            forms_df['filesSize'] = forms_df['filesSize'].apply(lambda x: self._human_friendly_filesize(int(x)))
-            forms_df.set_index('name',inplace=True)
+            forms_df = forms_df.append({"name": "Total:", "resultsCount": forms_df["resultsCount"].sum(), "filesSize": forms_df["filesSize"].sum(), "form_id": f"{forms_df.shape[0]} forms"}, ignore_index=True)
+            forms_df["filesSize"] = forms_df["filesSize"].apply(lambda x: self._human_friendly_filesize(int(x)))
+            forms_df.set_index("name", inplace=True)
             print(forms_df)
 
         if save2csv is not False:
             forms_df.sort_values(by=[sort_by], inplace=True, ascending=False)
-            forms_df.set_index('name',inplace=True)
+            forms_df.set_index("name", inplace=True)
             output_file = _validate_path(str(save2csv))
-            forms_df.to_csv(output_file, encoding='utf-8-sig')
+            forms_df.to_csv(output_file, encoding="utf-8-sig")
 
         return forms_df
 
-    def ReturnLinks(self, links_regex: str = r'.+') -> Set[str]:
+    def ReturnLinks(self, links_regex: str = r".+") -> Set[str]:
         """Stores a set of links in `self.Links` of files saved on formsite servers, that were submitted to the specified form.
 
         Args:
             links_filter_re (str, optional): Only keep links that match the regex pattern string.. Defaults to r'.+'.
         """
-        if self.Links is None or links_regex != r'.+':
+        if self.Links is None or links_regex != r".+":
             self.ExtractLinks(links_filter_re=links_regex)
         return self.Links
 
-    def WriteLinks(self,
-                   destination_path: str,
-                   links_regex: str = r'.+',
-                   sort_descending: bool = True) -> None:
+    def WriteLinks(self, destination_path: str, links_regex: str = r".+", sort_descending: bool = True) -> None:
         """Writes links extracted with `self.ExtractLinks()` to a .txt file.
 
         Args:
@@ -533,24 +521,17 @@ class FormsiteInterface:
             links_regex (str, optional): Include only links that match target regex. Defaults to r'.+'.
             sort_descending (bool, optional): Defaults to True.
         """
-        if self.Links is None or links_regex != r'.+':
+        if self.Links is None or links_regex != r".+":
             self.ExtractLinks(links_filter_re=links_regex)
         output_file = _validate_path(destination_path)
-        sorted_links = [link + '\n' for link in self.Links]
+        sorted_links = [link + "\n" for link in self.Links]
         sorted_links.sort(reverse=sort_descending)
-        with open(output_file, 'w') as writer:
+        with open(output_file, "w") as writer:
             writer.writelines(sorted_links)
 
-    def DownloadFiles(self,
-                      download_folder: str,
-                      max_concurrent_downloads: int = 10,
-                      links_regex: str = r'.+',
-                      filename_regex: str = r'',
-                      overwrite_existing: bool = True,
-                      report_downloads: bool = False,
-                      timeout: float = 80,
-                      retries: int = 1,
-                      strip_prefix: bool = False) -> None:
+    def DownloadFiles(
+        self, download_folder: str, max_concurrent_downloads: int = 10, links_regex: str = r".+", filename_regex: str = r"", overwrite_existing: bool = True, report_downloads: bool = False, timeout: float = 80, retries: int = 1, strip_prefix: bool = False
+    ) -> None:
         """Downloads files saved on formsite servers, that were submitted to the specified form.
         Please customize `max_concurrent_downloads` to your specific use case.
 
@@ -573,56 +554,50 @@ class FormsiteInterface:
         assert len(self.Links) > 0, f"There are no files to be downloaded for form {self.form_id}"
 
         download_folder = _validate_path(download_folder)
-        download_handler = _FormsiteDownloader(download_folder,
-                                               self.Links,
-                                               max_concurrent_downloads,
-                                               overwrite_existing=overwrite_existing,
-                                               filename_regex=filename_regex,
-                                               report_downloads=report_downloads,
-                                               timeout=timeout,
-                                               retries=retries,
-                                               display_progress=self.display_progress,
-                                               strip_prefix=strip_prefix)
+        download_handler = _FormsiteDownloader(
+            download_folder,
+            self.Links,
+            max_concurrent_downloads,
+            overwrite_existing=overwrite_existing,
+            filename_regex=filename_regex,
+            report_downloads=report_downloads,
+            timeout=timeout,
+            retries=retries,
+            display_progress=self.display_progress,
+            strip_prefix=strip_prefix,
+        )
         asyncio.get_event_loop().run_until_complete(download_handler.Start())
 
     def ListColumns(self, print_stdout: bool = True) -> pd.DataFrame:
         """Prints list of columns (items, usercontrols) and their respective formsite IDs."""
-        api_handler = _FormsiteAPI(form_id=self.form_id,
-                                   params=self.params,
-                                   auth=self.auth,
-                                   display_progress=self.display_progress)
+        api_handler = _FormsiteAPI(form_id=self.form_id, params=self.params, auth=self.auth, display_progress=self.display_progress)
         api_handler.check_pages = False
         items, _ = api_handler.Start(get_items=True, get_results=False)
-        items = pd.DataFrame(items['items'], columns=['id', 'label', 'position'])
+        items = pd.DataFrame(items["items"], columns=["id", "label", "position"])
         if print_stdout:
             print("id : label")
-            print('-----items----')
+            print("-----items----")
             items.apply(lambda row: print(f"{row['id']} : {row['label']}"), axis=1)
-            print('---metadata---')
-            print('id : Reference #')
-            print('result_status : Status')
-            print('date_start : Start Time')
-            print('date_finish : Finish Time')
-            print('date_update : Date')
-            print('user_ip : User')
-            print('user_browser : Browser')
-            print('user_device : Device')
-            print('user_referrer : Referrer')
-            print('payment_status : Payment Status')
-            print('payment_amount : Payment Amount Paid')
-            print('login_username : Login Username')
-            print('login_email : Login Email')
-            print('--parameters--')
+            print("---metadata---")
+            print("id : Reference #")
+            print("result_status : Status")
+            print("date_start : Start Time")
+            print("date_finish : Finish Time")
+            print("date_update : Date")
+            print("user_ip : User")
+            print("user_browser : Browser")
+            print("user_device : Device")
+            print("user_referrer : Referrer")
+            print("payment_status : Payment Status")
+            print("payment_amount : Payment Amount Paid")
+            print("login_username : Login Username")
+            print("login_email : Login Email")
+            print("--parameters--")
             print(f"Results labels: {self.params.resultslabels}")
             print(f"Results view: {self.params.resultsview}")
         return items
 
-    def WriteResults(self, destination_path: str,
-                     encoding: str = "utf-8-sig",
-                     line_terminator: str = 'os_default',
-                     separator: str = ",",
-                     date_format: str = "%Y-%m-%d %H:%M:%S",
-                     quoting: int = csv.QUOTE_MINIMAL) -> None:
+    def WriteResults(self, destination_path: str, encoding: str = "utf-8-sig", line_terminator: str = "os_default", separator: str = ",", date_format: str = "%Y-%m-%d %H:%M:%S", quoting: int = csv.QUOTE_MINIMAL) -> None:
         """Writes `self.Data` to a file based on provided extension.
         Supported output formats are (`.csv`|`.xlsx`|`.pkl`|`.pickle`|`.json`|`.parquet`|`.md`|`.txt`)
 
@@ -637,34 +612,31 @@ class FormsiteInterface:
         if self.Data is None:
             self.FetchResults()
         output_file = _validate_path(destination_path)
-        if re.search('.+\\.txt$', output_file) is not None:
+        if re.search(".+\\.txt$", output_file) is not None:
             self.Data.to_string(output_file, encoding=encoding, index=False)
-        elif re.search('.+\\.pkl$', output_file) is not None or re.search('.+\\.pickle$', output_file) is not None:
+        elif re.search(".+\\.pkl$", output_file) is not None or re.search(".+\\.pickle$", output_file) is not None:
             self.Data.to_pickle(output_file)
-        elif re.search('.+\\.parquet$', output_file) is not None:
+        elif re.search(".+\\.parquet$", output_file) is not None:
             self.Data.to_parquet(output_file, index=False)
-        elif re.search('.+\\.md$', output_file) is not None:
+        elif re.search(".+\\.md$", output_file) is not None:
             self.Data.to_markdown(output_file, index=False)
-        elif re.search('.+\\.json$', output_file) is not None:
+        elif re.search(".+\\.json$", output_file) is not None:
             try:
-                self.Data.to_json(output_file, orient='records', date_format='iso')
+                self.Data.to_json(output_file, orient="records", date_format="iso")
             except ValueError:
                 renamer: dict[Any, Any] = defaultdict()
                 for column_name in self.Data.columns[self.Data.columns.duplicated(keep=False)].tolist():
                     if column_name not in renamer:
-                        renamer[column_name] = [column_name+'_0']
+                        renamer[column_name] = [column_name + "_0"]
                     else:
-                        renamer[column_name].append(
-                            column_name + '_'+str(len(renamer[column_name])))
-                self.Data.rename(columns=lambda column_name: renamer[column_name].pop(
-                    0) if column_name in renamer else column_name).to_json(output_file, orient='records', date_format='iso')
-        elif re.search('.+\\.xlsx$', output_file) is not None:
-            print('Writing to excel (this can be slow for large files!)')
+                        renamer[column_name].append(column_name + "_" + str(len(renamer[column_name])))
+                self.Data.rename(columns=lambda column_name: renamer[column_name].pop(0) if column_name in renamer else column_name).to_json(output_file, orient="records", date_format="iso")
+        elif re.search(".+\\.xlsx$", output_file) is not None:
+            print("Writing to excel (this can be slow for large files!)")
             self.Data.to_excel(output_file, index=False)
         else:
-            line_terminator = os.linesep if line_terminator == 'os_default' else line_terminator
-            self.Data.to_csv(output_file, index=False, chunksize=1024, encoding=encoding,
-                             date_format=date_format, line_terminator=line_terminator, sep=separator, quoting=quoting)
+            line_terminator = os.linesep if line_terminator == "os_default" else line_terminator
+            self.Data.to_csv(output_file, index=False, chunksize=1024, encoding=encoding, date_format=date_format, line_terminator=line_terminator, sep=separator, quoting=quoting)
 
     def WriteLatestRef(self, destination_path: str) -> None:
         """Writes `max(self.Data['Reference #])` to a file.
@@ -679,12 +651,12 @@ class FormsiteInterface:
             self.FetchResults()
 
         output_file = _validate_path(destination_path)
-        if 'id' in self.Data.columns or 'Reference #' in self.Data.columns:
+        if "id" in self.Data.columns or "Reference #" in self.Data.columns:
             try:
-                latest_ref = max(self.Data['Reference #'])
+                latest_ref = max(self.Data["Reference #"])
             except KeyError:
-                latest_ref = max(self.Data['id'])
-            with open(output_file, 'w') as writer:
+                latest_ref = max(self.Data["id"])
+            with open(output_file, "w") as writer:
                 writer.write(str(latest_ref))
         else:
             raise KeyError("No valid column to extract Reference # from in specified parameters.")
